==> ctpp2-2.0.4.tar.gz: ctpp2-2.0.4/include/VM.hpp <==
/*-
 * Copyright (c) 2006, 2007 CTPP Team
 *
 * Redistribution and use in source and binary forms, with or without
 * modification, are permitted provided that the following conditions
 * are met:
 * 1. Redistributions of source code must retain the above copyright
 *    notice, this list of conditions and the following disclaimer.
 * 2. Redistributions in binary form must reproduce the above copyright
 *    notice, this list of conditions and the following disclaimer in the
 *    documentation and/or other materials provided with the distribution.
 * 4. Neither the name of the CTPP Team nor the names of its contributors
 *    may be used to endorse or promote products derived from this software
 *    without specific prior written permission.
 *
 * THIS SOFTWARE IS PROVIDED BY THE REGENTS AND CONTRIBUTORS ``AS IS'' AND
 * ANY EXPRESS OR IMPLIED WARRANTIES, INCLUDING, BUT NOT LIMITED TO, THE
 * IMPLIED WARRANTIES OF MERCHANTABILITY AND FITNESS FOR A PARTICULAR PURPOSE
 * ARE DISCLAIMED.  IN NO EVENT SHALL THE REGENTS OR CONTRIBUTORS BE LIABLE
 * FOR ANY DIRECT, INDIRECT, INCIDENTAL, SPECIAL, EXEMPLARY, OR CONSEQUENTIAL
 * DAMAGES (INCLUDING, BUT NOT LIMITED TO, PROCUREMENT OF SUBSTITUTE GOODS
 * OR SERVICES; LOSS OF USE, DATA, OR PROFITS; OR BUSINESS INTERRUPTION)
 * HOWEVER CAUSED AND ON ANY THEORY OF LIABILITY, WHETHER IN CONTRACT, STRICT
 * LIABILITY, OR TORT (INCLUDING NEGLIGENCE OR OTHERWISE) ARISING IN ANY WAY
 * OUT OF THE USE OF THIS SOFTWARE, EVEN IF ADVISED OF THE POSSIBILITY OF
 * SUCH DAMAGE.
 *
 *      VM.hpp
 *
 * $CTPP$
 */
#ifndef _VM_HPP__
#define _VM_HPP__ 1

/**
  @file VM.hpp
  @brief Virtual Machine
*/

#include <OutputCollector.hpp>
#include <SyscallFactory.hpp>
#include <VMArgStack.hpp>
#include <VMCodeStack.hpp>
#include <VMMemoryCore.hpp>

namespace CTPP // C++ Template Engine
{

// Aliases for registers
#define AR oRegs[0]
#define BR oRegs[1]
#define CR oRegs[2]
#define DR oRegs[3]
#define ER oRegs[4]
#define FR oRegs[5]
#define GR oRegs[6]
#define HR oRegs[7]


/**
  @class VM VM.hpp <VM.hpp>
  @brief Virtual Machine implementation
*/
class VM
{
public:
	/**
	  @brief Constructor
	  @param oISyscallFactory - factory with system calls
	  @param iIMaxArgStackSize - max. size of arguments stack
	  @param iIMaxCodeStackSize - max. size of code stack
	  @param iIMaxSteps - max. number of executed steps
	  @param iIDebugLevel - debugging level
	*/
	VM(SyscallFactory & oISyscallFactory,
           const UINT_32  & iIMaxArgStackSize  = 4096,
	   const UINT_32  & iIMaxCodeStackSize = 4096,
	   const UINT_32  & iIMaxSteps         = 10240,
	   const UINT_32  & iIDebugLevel       = 0);

	/**
	  @brief Initialize virtual machine
	*/
	INT_32 Init(OutputCollector     & pOutputCollector,
	            const VMMemoryCore  & oMemoryCore);

	/**
	  @brief Run program
	  @param oMemoryCore - ready-to-run core of program
	  @param iIP - instruction pointer
	  @param oCDT - initial data
	  @return stack depth
	*/
	INT_32 Run(const VMMemoryCore  & oMemoryCore,
	           UINT_32             & iIP,
	           CDT                 & oCDT);

//	/ **
//	  @brief Run one instruction
//	  @param oMemoryCore - ready-to-run core of program
//	  @param iIP - instruction pointer
//
//	  @return New instruction pointer
//	* /
//	INT_32 Step(VMInstruction * aCode, const UINT_32 & iCodeLength, UINT_32 & iIP);

	/**
	  @brief Reset virtual machine state
	*/
	INT_32 Reset();

	/**
	  @brief A destructor
	*/
	~VM() throw();

private:
	/** System calls factory         */
	SyscallFactory  & oSyscallFactory;
	/** Maximal arguments stack size */
	const UINT_32     iMaxArgStackSize;
	/** Maximal code stack size      */
	const UINT_32     iMaxCodeStackSize;
	/** Maximum program execution
	                           steps */
	const UINT_32     iMaxSteps;
	/** Debug level                  */
	const UINT_32     iDebugLevel;

	/** Number of system calls       */
	UINT_32           iMaxCalls;
	/** Number of system calls       */
	UINT_32           iMaxUsedCalls;
	/** System calls translation map */
	SyscallHandler ** aCallTranslationMap;

	/** Stack of arguments           */
	VMArgStack        oVMArgStack;
	/** Stack of code return points  */
	VMCodeStack       oVMCodeStack;

	/** Virtual machine registers    */
	CDT               oRegs[8];
	/** Virtual flags                */
	UINT_32           iFlags;
};

} // namespace CTPP
#endif // _VM_HPP__
// End.

==> ctpp2-2.4.1.tar.gz: ctpp2-2.4.1/include/VM.hpp <==
/*-
 * Copyright (c) 2004 - 2009 CTPP Team
 *
 * Redistribution and use in source and binary forms, with or without
 * modification, are permitted provided that the following conditions
 * are met:
 * 1. Redistributions of source code must retain the above copyright
 *    notice, this list of conditions and the following disclaimer.
 * 2. Redistributions in binary form must reproduce the above copyright
 *    notice, this list of conditions and the following disclaimer in the
 *    documentation and/or other materials provided with the distribution.
 * 4. Neither the name of the CTPP Team nor the names of its contributors
 *    may be used to endorse or promote products derived from this software
 *    without specific prior written permission.
 *
 * THIS SOFTWARE IS PROVIDED BY THE REGENTS AND CONTRIBUTORS ``AS IS'' AND
 * ANY EXPRESS OR IMPLIED WARRANTIES, INCLUDING, BUT NOT LIMITED TO, THE
 * IMPLIED WARRANTIES OF MERCHANTABILITY AND FITNESS FOR A PARTICULAR PURPOSE
 * ARE DISCLAIMED.  IN NO EVENT SHALL THE REGENTS OR CONTRIBUTORS BE LIABLE
 * FOR ANY DIRECT, INDIRECT, INCIDENTAL, SPECIAL, EXEMPLARY, OR CONSEQUENTIAL
 * DAMAGES (INCLUDING, BUT NOT LIMITED TO, PROCUREMENT OF SUBSTITUTE GOODS
 * OR SERVICES; LOSS OF USE, DATA, OR PROFITS; OR BUSINESS INTERRUPTION)
 * HOWEVER CAUSED AND ON ANY THEORY OF LIABILITY, WHETHER IN CONTRACT, STRICT
 * LIABILITY, OR TORT (INCLUDING NEGLIGENCE OR OTHERWISE) ARISING IN ANY WAY
 * OUT OF THE USE OF THIS SOFTWARE, EVEN IF ADVISED OF THE POSSIBILITY OF
 * SUCH DAMAGE.
 *
 *      VM.hpp
 *
 * $CTPP$
 */
#ifndef _VM_HPP__
#define _VM_HPP__ 1

/**
  @file VM.hpp
  @brief Virtual Machine
*/

#include <OutputCollector.hpp>
#include <SyscallFactory.hpp>
#include <VMArgStack.hpp>
#include <VMCodeStack.hpp>
#include <VMMemoryCore.hpp>

namespace CTPP // C++ Template Engine
{

// Aliases for registers
#define AR oRegs[0]
#define BR oRegs[1]
#define CR oRegs[2]
#define DR oRegs[3]
#define ER oRegs[4]
#define FR oRegs[5]
#define GR oRegs[6]
#define HR oRegs[7]


/**
  @class VM VM.hpp <VM.hpp>
  @brief Virtual Machine implementation
*/
class VM
{
public:
	/**
	  @brief Constructor
	  @param oISyscallFactory - factory with system calls
	  @param iIMaxArgStackSize - max. size of arguments stack
	  @param iIMaxCodeStackSize - max. size of code stack
	  @param iIMaxSteps - max. number of executed steps
	  @param iIDebugLevel - debugging level
	*/
	VM(SyscallFactory & oISyscallFactory,
           const UINT_32  & iIMaxArgStackSize  = 4096,
	   const UINT_32  & iIMaxCodeStackSize = 4096,
	   const UINT_32  & iIMaxSteps         = 10240,
	   const UINT_32  & iIDebugLevel       = 0);

	/**
	  @brief Initialize virtual machine
	*/
	INT_32 Init(OutputCollector     & pIOutputCollector,
	            const VMMemoryCore  & oMemoryCore);

	/**
	  @brief Run program
	  @param oMemoryCore - ready-to-run core of program
	  @param iIP - instruction pointer
	  @param oCDT - initial data
	  @return stack depth
	*/
	INT_32 Run(const VMMemoryCore  & oMemoryCore,
	           UINT_32             & iIP,
	           CDT                 & oCDT);

//	/ **
//	  @brief Run one instruction
//	  @param oMemoryCore - ready-to-run core of program
//	  @param iIP - instruction pointer
//
//	  @return New instruction pointer
//	* /
//	INT_32 Step(VMInstruction * aCode, const UINT_32 & iCodeLength, UINT_32 & iIP);

	/**
	  @brief Reset virtual machine state
	*/
	INT_32 Reset();

	/**
	  @brief A destructor
	*/
	~VM() throw();

private:
	/** System calls factory         */
	SyscallFactory   & oSyscallFactory;
	/** Maximal arguments stack size */
	const UINT_32      iMaxArgStackSize;
	/** Maximal code stack size      */
	const UINT_32      iMaxCodeStackSize;
	/** Maximum program execution
	                           steps */
	const UINT_32      iMaxSteps;
	/** Debug level                  */
	const UINT_32      iDebugLevel;

	/** Output data collector        */
	OutputCollector  * pOutputCollector;

	/** Number of system calls       */
	UINT_32            iMaxCalls;
	/** Number of system calls       */
	UINT_32            iMaxUsedCalls;
	/** System calls translation map */
	SyscallHandler  ** aCallTranslationMap;

	/** Stack of arguments           */
	VMArgStack         oVMArgStack;
	/** Stack of code return points  */
	VMCodeStack        oVMCodeStack;

	/** Virtual machine registers    */
	CDT                oRegs[8];
	/** Virtual flags                */
	UINT_32            iFlags;
};

} // namespace CTPP
#endif // _VM_HPP__
// End.
